@@ -1,7 +1,7 @@
-/* $Id: download.c,v 1.6 2009/05/18 14:34:35 imil Exp $ */
+/* $Id: download.c,v 1.15 2010/01/03 14:07:29 imil Exp $ */
 
 /*
- * Copyright (c) 2009 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -41,22 +41,41 @@ download_file(char *url, time_t *db_mtime)
 	size_t			buf_len, buf_fetched;
 	ssize_t			cur_fetched;
 	struct url_stat	st;
-	fetchIO			*f;
+	fetchIO			*f = NULL;
+	int				retry = 3;
 
-	if (fetchStatURL(url, &st, "") < 0)
-		errx(EXIT_FAILURE, "could not stat url: %s: %s",
-			url, fetchLastErrString);
+	if ((fetchStatURL(url, &st, "") < 0) || st.size == -1) {
+		if (db_mtime != NULL)
+			*db_mtime = 0;
+
+		return NULL;
+	}
 
 	if (db_mtime != NULL) {
-		if (st.mtime <= *db_mtime)
+		if (st.mtime <= *db_mtime) {
+			/* local db is up-to-date */
+			*db_mtime = -1; /* used to identify return type */
+
 			return NULL;
+		}
 
 		*db_mtime = st.mtime;
 	}
 
-	if ((f = fetchXGetURL(url, &st, "")) == NULL)
-		errx(EXIT_FAILURE, "could not fetch url: %s: %s",
-			url, fetchLastErrString);
+	while (f == NULL) {
+		f = fetchXGetURL(url, &st, "");
+		if (f == NULL) {
+			fprintf(stderr, "%s: fetch failure (%s)%s\n",
+				getprogname(), fetchLastErrString,
+				retry == 1 ? "" : ", retrying");
+			sleep(1);
+			--retry;
+		}
+		if (retry == 0)
+			errx(EXIT_FAILURE, "could not fetch url: %s: %s",
+				url, fetchLastErrString);
+	}
+
 
 	if ((p = strrchr(url, '/')) != NULL)
 		p++;
@@ -65,16 +84,17 @@ download_file(char *url, time_t *db_mtime)
 
 	if (st.size > SSIZE_MAX - 1)
 		err(EXIT_FAILURE, "file is too large");
-	
+
 	buf_len = st.size;
 	XMALLOC(file, sizeof(Dlfile));
 	if ((file->buf = malloc(buf_len + 1)) == NULL)
 		err(EXIT_FAILURE, "malloc failed");
-	
-	printf("\e[2K\e[?25l"); /* clear line */
+
+	printf(MSG_DOWNLOADING, p);
+	fflush(stdout);
 
 	buf_fetched = 0;
-	while (buf_fetched < buf_len) {		
+	while (buf_fetched < buf_len) {
 		cur_fetched = fetchIO_read(f, file->buf + buf_fetched,
 			buf_len - buf_fetched);
 		if (cur_fetched == 0)
@@ -84,19 +104,20 @@ download_file(char *url, time_t *db_mtime)
 				fetchLastErrString);
 		buf_fetched += cur_fetched;
 
-		printf(MSG_DOWNLOADING, p,
-			((float)buf_fetched / (float)buf_len) * 100);
+		printf(MSG_DOWNLOADING_PCT,
+			(int)(((float)buf_fetched / (float)buf_len) * 100));
+		fflush(stdout);
 	}
 
 	fetchIO_close(f);
-	
+
 	file->buf[buf_len] = '\0';
 	file->size = buf_len;
-	
+
 	if (file->buf[0] == '\0')
 		errx(EXIT_FAILURE, "empty download, exiting.\n");
 
-	printf("\n\e[?25h");
-	
+	printf("\n");
+
 	return file;
 }
