@@ -1,7 +1,7 @@
-/* $Id: fsops.c,v 1.2 2009/06/05 17:37:39 imil Exp $ */
+/* $Id: fsops.c,v 1.6 2010/01/03 14:07:29 imil Exp $ */
 
 /*
- * Copyright (c) 2009 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -33,6 +33,16 @@
 #include "pkgin.h"
 #include <dirent.h>
 
+/* Variable options for the repositories file */
+static const struct VarParam {
+	const char *opt;
+	char *(*func)(void);
+} var[] = {
+	{ "$arch", getosarch },
+	{ "$osrelease", getosrelease },
+	{ NULL, NULL }
+};
+
 int
 fs_has_room(const char *dir, int64_t size)
 {
@@ -56,7 +66,7 @@ clean_cache()
 	DIR				*dp;
 	struct dirent	*ep;
 	char			pkgpath[BUFSIZ];
-     
+
 	if ((dp = opendir(pkgin_cache)) != NULL) {
 		while ((ep = readdir(dp)) != NULL)
 			if (ep->d_name[0] != '.') {
@@ -66,7 +76,7 @@ clean_cache()
 			}
 		closedir(dp);
 	} else
-		err(EXIT_FAILURE, "couldn't open %s", pkgin_cache);	
+		err(EXIT_FAILURE, "couldn't open %s", pkgin_cache);
 }
 
 void
@@ -82,8 +92,9 @@ char *
 read_repos()
 {
 	FILE	*fp;
-	char	*repos = NULL, buf[BUFSIZ];
-	int		curlen = 0, repolen = 0;
+	char	*tmp, *b, *repos = NULL, buf[BUFSIZ];
+	int     curlen = 0, repolen = 0;
+	const struct VarParam	*vp;
 
 	if ((fp = fopen(PKGIN_CONF"/"REPOS_FILE, "r")) == NULL)
 		return NULL;
@@ -99,6 +110,28 @@ read_repos()
 			strncmp(buf, "file://", 7) != 0)
 			continue;
 
+		/*
+		 * Try to replace all the ocurrences with the proper
+		 * system values.
+		 */
+		for (vp = var; vp->func != NULL; vp++) {
+			if (strstr(buf, vp->opt) != NULL) {
+				tmp = vp->func();
+				if (tmp == NULL) {
+					warn(MSG_TRANS_FAILED, vp->opt);
+					continue;
+				}
+				if ((b = strreplace(buf, vp->opt, tmp)) != NULL)
+					strncpy(buf, b, BUFSIZ);
+				else
+					warn(MSG_INVALID_REPOS, buf);
+
+				XFREE(tmp);
+				XFREE(b);
+			}
+		}
+
+
 		if (trimcr(buf) < 0) /* strip newline */
 			continue;
 
