@@ -1,4 +1,4 @@
-/* $Id: order.c,v 1.13 2010/01/23 14:30:07 imil Exp $ */
+/* $Id: order.c,v 1.14 2010/01/24 17:02:03 imil Exp $ */
 
 /*
  * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
@@ -55,7 +55,7 @@ remove_dep_deepness(Deptreehead *deptreehead)
 		}
 
 		pdp->level = 1;
-
+		
 		if (pdp->depname == NULL)
 			/* there's something wrong with database's record, probably
 			 * a mistaken dependency
@@ -85,20 +85,32 @@ remove_dep_deepness(Deptreehead *deptreehead)
 Deptreehead *
 order_remove(Deptreehead *deptreehead)
 {
-	Pkgdeptree *pdp;
+	int i, maxlevel = 0;
+	Pkgdeptree *pdp, *next;
 	Deptreehead *ordtreehead;
 
 	/* package removal cannot trust recorded dependencies, reorder */
 	remove_dep_deepness(deptreehead);
 
+	SLIST_FOREACH(pdp, deptreehead, next)
+		if (pdp->level > maxlevel)
+			maxlevel = pdp->level;
+
 	XMALLOC(ordtreehead, sizeof(Deptreehead));
 	SLIST_INIT(ordtreehead);
 
-	while (!SLIST_EMPTY(deptreehead)) {
+	for (i = maxlevel; i >= 0; i--) {
 		pdp = SLIST_FIRST(deptreehead);
-		SLIST_REMOVE_HEAD(deptreehead, next);
-		SLIST_INSERT_HEAD(ordtreehead, pdp, next);
+		while (pdp != NULL) {
+			next = SLIST_NEXT(pdp, next);
+			if (pdp->level == i) {
+				SLIST_REMOVE(deptreehead, pdp, Pkgdeptree, next);
+				SLIST_INSERT_HEAD(ordtreehead, pdp, next);
+			}
+			pdp = next;
+		}
 	}
+
 	return ordtreehead;
 }
 
